local VERSION = tonumber(("$Revision$"):match("%d+"))
local DURATION = 0.5

local L = AceLibrary("AceLocale-2.2"):new("PitBull_BarFader")
L:RegisterTranslations("enUS", function() return {
	["BarFader"] = true,
	["Ignore Unit(s)"] = true,
} end)
local PitBull = PitBull
local PitBull_BarFader = PitBull:NewModule("BarFader", "AceHook-2.1", "AceEvent-2.0")
local _newFrame, _delFrame = PitBull.newFrame, PitBull.delFrame
local _PitBull_Health = PitBull:GetModule("Health")
local _PitBull_Mana = PitBull:GetModule("Mana")
local _wrap = PitBull.wrap
local _frames = {}

local function _onUpdate()
	local now, alpha, stop, bar = GetTime(), 1
	for bar,field in pairs(_frames) do
		stop = bar.faderstop
		alpha = ((stop - now) / DURATION)
		if (alpha <= 0) then
			if (bar:GetParent()[field] == bar) then -- make sure we didn't get reassigned somehow
				bar:SetAlpha(0)
				bar:Hide()
				bar.orientation = nil
				bar.stop = nil
				bar:GetParent()[field] = _delFrame(bar)
			end
			_frames[bar] = nil
		else
			bar:SetAlpha(alpha)
		end
	end
end

function PitBull_BarFader:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("BarFader")
	PitBull:RegisterDefaults("BarFader", "profile", {
		color = {0, 0.8, 0, 0.5},
		groups = {
			['*'] = {},
		}
	})
end

function PitBull_BarFader:OnEnable()
	self:Hook(_PitBull_Mana, "UpdateMana")
	self:Hook(_PitBull_Health, "UpdateHealth")
end

function PitBull_BarFader:PositionHealthBar(frame)
	frame.healthFade:ClearAllPoints()
	if frame.healthFade.orientation == "HORIZONTAL" then
		frame.healthFade:SetPoint("TOPLEFT", frame.healthBar.bg, "TOPLEFT", 0, 0)
	else
		frame.healthFade:SetPoint("BOTTOMLEFT", frame.healthBar.bg, "BOTTOMLEFT", 0, 0)
	end
end

function PitBull_BarFader:OnClearUnitFrame(unit, frame)
	if frame.healthFade then
		_frames[frame.healthFade] = nil
		frame.healthFade = _delFrame(frame.healthFade)
	end
	if frame.manaFade then
		_frames[frame.manaFade] = nil
		frame.manaFade = _delFrame(frame.manaFade)
	end
end

function PitBull_BarFader:PositionManaBar(frame)
	frame.manaFade:ClearAllPoints()
	if frame.manaFade.orientation == "HORIZONTAL" then
		frame.manaFade:SetPoint("TOPLEFT", frame.manaBar.bg, "TOPLEFT", 0, 0)
	else
		frame.manaFade:SetPoint("BOTTOMLEFT", frame.manaBar.bg, "BOTTOMLEFT", 0, 0)
	end
end

function PitBull_BarFader:UpdateHealth(object, unit, frame)
	local healthvalue = frame.healthBar:GetValue()
	self.hooks[object].UpdateHealth(object,unit,frame)
	if (frame.healthBar:GetValue() < healthvalue) then
		local healthFade
		if (frame.healthFade) then
			healthFade = frame.healthFade
		else
			healthFade = _newFrame("StatusBar", frame)
		end
		frame.healthFade = healthFade
		healthFade:SetWidth(frame.healthBar:GetWidth())
		healthFade:SetHeight(frame.healthBar:GetHeight())
		healthFade.orientation = frame.healthBar:GetOrientation()
		healthFade:SetOrientation(healthFade.orientation)
		
		local min, max = frame.healthBar:GetMinMaxValues()
		healthFade:SetMinMaxValues(min, max)
		healthFade:SetValue(healthvalue)
		healthFade:SetStatusBarTexture(PitBull:GetStatusBarTexture())
		local r, g, b = frame.healthBar:GetStatusBarColor()
		healthFade:SetStatusBarColor(r + 0.2, g + 0.2, b + 0.2)
		healthFade.faderstop = GetTime() + DURATION
		_frames[healthFade] = 'healthFade'
		healthFade:Show()
		self:PositionHealthBar(frame)
	end
end

function PitBull_BarFader:UpdateMana(object, unit, frame)
	local manavalue = frame.manaBar:GetValue()
	self.hooks[object].UpdateMana(object,unit,frame)
	if (frame.manaBar:GetValue() < manavalue) then
		local manaFade
		if (frame.manaFade) then
			manaFade = frame.manaFade
		else
			manaFade = _newFrame("StatusBar", frame)
		end
		frame.manaFade = manaFade
		manaFade:SetWidth(frame.manaBar:GetWidth())
		manaFade:SetHeight(frame.manaBar:GetHeight())
		manaFade.orientation = frame.manaBar:GetOrientation()
		manaFade:SetOrientation(manaFade.orientation)
		
		local min, max = frame.manaBar:GetMinMaxValues()
		manaFade:SetMinMaxValues(min, max)
		manaFade:SetValue(manavalue)
		manaFade:SetStatusBarTexture(PitBull:GetStatusBarTexture())
		local r, g, b = frame.manaBar:GetStatusBarColor()
		manaFade:SetStatusBarColor(r + 0.2, g + 0.2, b + 0.2)
		manaFade.faderstop = GetTime() + DURATION
		_frames[manaFade] = 'manaFade'
		manaFade:Show()
		self:PositionManaBar(frame)
	end
end

-- Global Options
local function getColor()
	return unpack(PitBull_BarFader.db.profile.color)
end
local function setColor(r,g,b,a)
	PitBull_BarFader.db.profile.color = {r,g,b,a}
end

-- Unit/Group Specific Options
local function getIgnored(group)
	return PitBull_BarFader.db.profile.groups[group].ignore
end
local function setIgnored(group, value)
	PitBull_BarFader.db.profile.groups[group].ignore = value
end
PitBull_BarFader:RegisterPitBullOptionsMethod(function(group)
	return {
		name = L["BarFader"],
		desc = L["BarFader"],
		type = 'group',
		args = {
			ignore = {
				type = 'toggle',
				name = L["Ignore Unit(s)"],
				desc = L["Ignore Unit(s)"],
				get = getIgnored,
				set = setIgnored,
				passValue = group,
			},
		}
	}
end)

_wrap(PitBull_BarFader, 'BarFader')

local f = CreateFrame("Frame", nil, UIParent)
f:SetScript("OnUpdate",_onUpdate)
